"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.withTransaction = void 0;
const withTransaction = async (config) => {
    var _a, _b;
    const isLocal = ((_b = (_a = process.env) === null || _a === void 0 ? void 0 : _a.NODE_ENV) === null || _b === void 0 ? void 0 : _b.toLowerCase()) === 'local';
    const transactionOptions = {
        readConcern: { level: 'majority' },
        writeConcern: { w: 'majority' },
        readPreference: 'primary',
        maxCommitTimeMS: 300000,
        causalConsistency: true,
    };
    const allSessions = [];
    const withChildTransaction = async (sessionConfigs, ...store) => {
        if (!sessionConfigs || !sessionConfigs.length)
            return true;
        return Promise.all(sessionConfigs.map(async (sessionConfig) => {
            const { connection, action, childrenSessions } = sessionConfig;
            const childSession = await connection.startSession();
            allSessions.push(childSession);
            await childSession.withTransaction(async () => {
                const result = await (action === null || action === void 0 ? void 0 : action(store, isLocal ? undefined : childSession));
                store.push(result);
                await withChildTransaction(childrenSessions || [], ...store);
                return true;
            }, transactionOptions);
        }));
    };
    try {
        const { connection, action, childrenSessions } = config;
        const session = await connection.startSession();
        allSessions.push(session);
        await session.withTransaction(async () => {
            const result = await (action === null || action === void 0 ? void 0 : action([], isLocal ? undefined : session));
            await withChildTransaction(childrenSessions || [], result);
            return true;
        }, transactionOptions);
    }
    finally {
        // End the sessions and close the clients
        await Promise.all(allSessions.map((ses_) => ses_.endSession()));
    }
};
exports.withTransaction = withTransaction;
//# sourceMappingURL=index.js.map